#ifndef FLOWCALCULATOR_H
#define FLOWCALCULATOR_H

#include "../Centrality/CentralityCalculator.cxx"
#include "../Utils/BaseReader.cxx"

#include <TVector2.h>
#include <TRandom3.h>
#include <TDatime.h>
#include <vector>

class FlowCalculator : public BaseReader
{
public:
    CentralityCalculator *centCalc;
    std::vector<Double_t> eta_gap;
    std::vector<std::pair<Double_t,Double_t> > eta_rxn;

    TH1F *h_res2TPC[4];
    TH1F *h_res3TPC[4];
    TH1F *h_res2RXN[3];
    TH1F *h_res3RXN[3];

    TVector2 fQv2EastTPC[4], fQv3EastTPC[4];
    TVector2 fQv2WestTPC[4], fQv3WestTPC[4];
    TVector2 fQv2EastRXN[3], fQv3EastRXN[3];
    TVector2 fQv2WestRXN[3], fQv3WestRXN[3];
    TVector2 fQv2FullRXN[3], fQv3FullRXN[3];
    TVector2 fQv1EastZDC, fQv1WestZDC, fQv1FullZDC;
    TVector2 fQv2MiddTPC, fQv3MiddTPC;
    int multEastRXN[3], multWestRXN[3];
    int multEastZDC, multWestZDC;

    Double_t fRes2RXN[3][10];
    Double_t fRes2TPC[3][10];

    Double_t fRes3RXN[3][10];
    Double_t fRes3TPC[3][10];

    TString histRefMultName;
    Bool_t  fRotate;
    TRandom3 *fRnd;
    TDatime  *fDate;
    Float_t  fPsiRP;

    FlowCalculator(TTree *tree=0);
    virtual ~FlowCalculator();

    void InitCentFile(TString inName);
    void InitResFile(TString inName);
    void LoopRes(TString outName);
    void LoopFlow(TString outName);
    TVector2 GetQvTPC(Int_t side, Int_t harm, Int_t gap);
    TVector2 GetQvBBC(Int_t side, Int_t harm);
    TVector2 GetQvRXN(Int_t side, Int_t harm, Int_t part);
    void     GetAllQv();
    Double_t GetPsiEP(const TVector2 &qv, Double_t harm);
    void     SetRefMultHistName(TString _name) { histRefMultName = _name; }
    void     RotateRP() { fRotate = true; }
};

#endif
