#include <TGraphErrors.h>
#include <TFile.h>
#include <TH1D.h>
#include <TProfile.h>
#include <TString.h>

#include <iostream>

TGraphErrors *MakeGraph(TH1D *const& hist)
{
    TGraphErrors *gr;
    std::vector<Double_t> vX, vY, vEx, vEy;

    for (int ibin=0; ibin<hist->GetNbinsX(); ibin++)
    {
        vX.push_back(hist->GetBinCenter(ibin+1));
        vY.push_back(hist->GetBinContent(ibin+1));
        vEx.push_back(0.);
        vEy.push_back(hist->GetBinError(ibin+1));
    }
    gr = new TGraphErrors(vX.size(),&vX[0],&vY[0],&vEx[0],&vEy[0]);
    return gr;
}

void SaveGraphs(TString inName, TString outName)
{
    const Int_t NptBins = 8;
    const Double_t ptBins[NptBins+1] = {0.2,0.4,0.6,0.8,1.,1.2,1.8,2.3,2.8};
    const Int_t NcentBins = 8;
    const std::pair<Int_t,Int_t> centRange[NcentBins] = {{0,0},{1,1},{2,2},{3,3},{4,4},{5,5},{3,5},{0,5}}; // 0-10%, 10-20%, 20-30%, 30-40%, 40-50%, 50-60%, 30-50%, 0-50%
    const TString centName [NcentBins] = {TString("0-10%"), TString("10-20%"), TString("20-30%"),
        TString("30-40%"), TString("40-50%"), TString("50-60%"), TString("30-50%"), TString("0-50%")};
    const double etaGap [4] = {0.1,0.2,0.3,0.5};
    const TString pidName [4] = {TString("h^{#pm}"),TString("#pi^{#pm}"),TString("K^{#pm}"),TString("p+#bar{p}")};
    TFile *fi = new TFile(inName.Data(),"read");

    TProfile *p_v2TPC[4][10][4], *p_v3TPC[4][10][4];
    TProfile *p_v2RXN[3][10][4], *p_v3RXN[3][10][4];
    // [4/3][10][4] -- EtaGaps/RXN parts, CentralityBins, PID
    // PID: 0 - hadrons, 1 - pions, 2 - kaons, 3 - protons
    for (int iGap=0; iGap<4; iGap++)
    {   
        for (int iCent=0; iCent<10; iCent++)
        {
            for (int iPID=0; iPID<4; iPID++)
            {
                p_v2TPC[iGap][iCent][iPID] = (TProfile*) fi->Get(Form("p_v2TPC_gap%i_cent%i_pid%i",iGap,iCent,iPID));
                p_v3TPC[iGap][iCent][iPID] = (TProfile*) fi->Get(Form("p_v3TPC_gap%i_cent%i_pid%i",iGap,iCent,iPID));
            }
        }
    }
    for (int irxn=0; irxn<3; irxn++)
    {   
        for (int iCent=0; iCent<10; iCent++)
        {
            for (int iPID=0; iPID<4; iPID++)
            {
                p_v2RXN[irxn][iCent][iPID] = (TProfile*) fi->Get(Form("p_v2RXN_part%i_cent%i_pid%i",irxn,iCent,iPID));
                p_v3RXN[irxn][iCent][iPID] = (TProfile*) fi->Get(Form("p_v3RXN_part%i_cent%i_pid%i",irxn,iCent,iPID));
            }
        }
    }

    TProfile *prof_v2_TPC[4][NcentBins][4], *prof_v3_TPC[4][NcentBins][4];
    TH1D     *h_v2_TPC[4][NcentBins][4], *h_v3_TPC[4][NcentBins][4];
    TGraphErrors *gr_v2_TPC[4][NcentBins][4], *gr_v3_TPC[4][NcentBins][4];

    TProfile *prof_v2_RXN[3][NcentBins][4], *prof_v3_RXN[3][NcentBins][4];
    TH1D     *h_v2_RXN[3][NcentBins][4], *h_v3_RXN[3][NcentBins][4];
    TGraphErrors *gr_v2_RXN[3][NcentBins][4], *gr_v3_RXN[3][NcentBins][4];

    for (int iGap=0; iGap<4; iGap++)
    {   
        for (int iC=0; iC<NcentBins; iC++)
        {
            for (int iPID=0; iPID<4; iPID++)
            {
                prof_v2_TPC[iGap][iC][iPID] = (TProfile*) p_v2TPC[iGap][centRange[iC].first][iPID]->Clone();
                prof_v3_TPC[iGap][iC][iPID] = (TProfile*) p_v3TPC[iGap][centRange[iC].first][iPID]->Clone();
                for (int icc=centRange[iC].first; icc<centRange[iC].second;icc++)
                {
                    prof_v2_TPC[iGap][iC][iPID]->Add(p_v2TPC[iGap][icc+1][iPID]);
                    prof_v3_TPC[iGap][iC][iPID]->Add(p_v3TPC[iGap][icc+1][iPID]);
                }
            }
        }
    }
    for (int irxn=0; irxn<3; irxn++)
    {   
        for (int iC=0; iC<NcentBins; iC++)
        {
            for (int iPID=0; iPID<4; iPID++)
            {
                prof_v2_RXN[irxn][iC][iPID] = (TProfile*) p_v2RXN[irxn][centRange[iC].first][iPID]->Clone();
                prof_v3_RXN[irxn][iC][iPID] = (TProfile*) p_v3RXN[irxn][centRange[iC].first][iPID]->Clone();
                for (int icc=centRange[iC].first; icc<centRange[iC].second;icc++)
                {
                    prof_v2_RXN[irxn][iC][iPID]->Add(p_v2RXN[irxn][icc+1][iPID]);
                    prof_v3_RXN[irxn][iC][iPID]->Add(p_v3RXN[irxn][icc+1][iPID]);
                }
            }
        }
    }

    for (int iGap=0; iGap<4; iGap++)
    {   
        for (int iC=0; iC<NcentBins; iC++)
        {
            for (int iPID=0; iPID<4; iPID++)
            {
                h_v2_TPC[iGap][iC][iPID] = (TH1D*) prof_v2_TPC[iGap][iC][iPID]->Rebin(NptBins,Form("h_v2_TPC_%i_%i_%i",iGap,iC,iPID),ptBins);
                h_v3_TPC[iGap][iC][iPID] = (TH1D*) prof_v3_TPC[iGap][iC][iPID]->Rebin(NptBins,Form("h_v3_TPC_%i_%i_%i",iGap,iC,iPID),ptBins);
            }
        }
    }

    for (int irxn=0; irxn<3; irxn++)
    {   
        for (int iC=0; iC<NcentBins; iC++)
        {
            for (int iPID=0; iPID<4; iPID++)
            {
                h_v2_RXN[irxn][iC][iPID] = (TH1D*) prof_v2_RXN[irxn][iC][iPID]->Rebin(NptBins,Form("h_v2_RXN_%i_%i_%i",irxn,iC,iPID),ptBins);
                h_v3_RXN[irxn][iC][iPID] = (TH1D*) prof_v3_RXN[irxn][iC][iPID]->Rebin(NptBins,Form("h_v3_RXN_%i_%i_%i",irxn,iC,iPID),ptBins);
            }
        }
    }

    for (int iGap=0; iGap<4; iGap++)
    {   
        for (int iC=0; iC<NcentBins; iC++)
        {
            for (int iPID=0; iPID<4; iPID++)
            {
                gr_v2_TPC[iGap][iC][iPID] = (TGraphErrors*) MakeGraph(h_v2_TPC[iGap][iC][iPID]);
                gr_v2_TPC[iGap][iC][iPID]->SetName(Form("grv2TPC_gap%i_cent%i_pid%i",iGap,iC,iPID));
                gr_v2_TPC[iGap][iC][iPID]->SetTitle(Form("v_{2}(TPC,#Delta#eta-gap=%1.1f) %s, %s;p_{T}, [GeV/c];v_{2}",etaGap[iGap],pidName[iPID].Data(),centName[iC].Data()));
                gr_v3_TPC[iGap][iC][iPID] = (TGraphErrors*) MakeGraph(h_v3_TPC[iGap][iC][iPID]);
                gr_v3_TPC[iGap][iC][iPID]->SetName(Form("grv3TPC_gap%i_cent%i_pid%i",iGap,iC,iPID));
                gr_v3_TPC[iGap][iC][iPID]->SetTitle(Form("v_{3}(TPC,#Delta#eta-gap=%1.1f) %s, %s;p_{T}, [GeV/c];v_{3}",etaGap[iGap],pidName[iPID].Data(),centName[iC].Data()));
            }
        }
    }

    for (int irxn=0; irxn<3; irxn++)
    {   
        for (int iC=0; iC<NcentBins; iC++)
        {
            for (int iPID=0; iPID<4; iPID++)
            {
                gr_v2_RXN[irxn][iC][iPID] = (TGraphErrors*) MakeGraph(h_v2_RXN[irxn][iC][iPID]);
                gr_v2_RXN[irxn][iC][iPID]->SetName(Form("grv2RXN_part%i_cent%i_pid%i",irxn,iC,iPID));
                gr_v2_RXN[irxn][iC][iPID]->SetTitle(Form("v_{2}(RXN,part %i) %s, %s;p_{T}, [GeV/c];v_{2}",irxn,pidName[iPID].Data(),centName[iC].Data()));
                gr_v3_RXN[irxn][iC][iPID] = (TGraphErrors*) MakeGraph(h_v3_RXN[irxn][iC][iPID]);
                gr_v3_RXN[irxn][iC][iPID]->SetName(Form("grv3RXN_part%i_cent%i_pid%i",irxn,iC,iPID));
                gr_v3_RXN[irxn][iC][iPID]->SetTitle(Form("v_{3}(RXN,part %i) %s, %s;p_{T}, [GeV/c];v_{3}",irxn,pidName[iPID].Data(),centName[iC].Data()));
            }
        }
    }

    TFile *fo = new TFile(outName.Data(),"recreate");
    fo->cd();
    for (int iGap=0; iGap<4; iGap++)
    {   
        for (int iC=0; iC<NcentBins; iC++)
        {
            for (int iPID=0; iPID<4; iPID++)
            {
                gr_v2_TPC[iGap][iC][iPID]->Write();
                gr_v3_TPC[iGap][iC][iPID]->Write();
            }
        }
    }
    for (int irxn=0; irxn<3; irxn++)
    {   
        for (int iC=0; iC<NcentBins; iC++)
        {
            for (int iPID=0; iPID<4; iPID++)
            {
                gr_v2_RXN[irxn][iC][iPID]->Write();
                gr_v3_RXN[irxn][iC][iPID]->Write();
            }
        }
    }
    fo->Close();
}