#ifndef BaseReader_cxx
#define BaseReader_cxx
#include "BaseReader.h"
#include <TH2.h>
#include <TStyle.h>
#include <TCanvas.h>

void BaseReader::Loop()
{
//   In a ROOT session, you can do:
//      root> .L BaseReader.C
//      root> BaseReader t
//      root> t.GetEntry(12); // Fill t data members with entry number 12
//      root> t.Show();       // Show values of entry 12
//      root> t.Show(16);     // Read and show values of entry 16
//      root> t.Loop();       // Loop on all entries
//

//     This is the loop skeleton where:
//    jentry is the global entry number in the chain
//    ientry is the entry number in the current Tree
//  Note that the argument to GetEntry must be:
//    jentry for TChain::GetEntry
//    ientry for TTree::GetEntry and TBranch::GetEntry
//
//       To read only selected branches, Insert statements like:
// METHOD1:
//    fChain->SetBranchStatus("*",0);  // disable all branches
//    fChain->SetBranchStatus("branchname",1);  // activate branchname
// METHOD2: replace line
//    fChain->GetEntry(jentry);       //read all branches
//by  b_branchname->GetEntry(ientry); //read only this branch
   if (fChain == 0) return;

   Long64_t nentries = fChain->GetEntries();

   Long64_t nbytes = 0, nb = 0;
   for (Long64_t jentry=0; jentry<nentries;jentry++) {
      Long64_t ientry = LoadTree(jentry);
      if (ientry < 0) break;
      nb = fChain->GetEntry(jentry);   nbytes += nb;
      // if (Cut(ientry) < 0) continue;
   }
}


Int_t BaseReader::GetMult(Int_t iEv)
{
   Int_t refMult = 0;
   if (iEv != -1)
   {
      Long64_t ientry = LoadTree(iEv);
      if (ientry < 0) return -1;
      fChain->GetEntry(iEv);
      // if (Cut(ientry) < 0) continue;
   }

   for (int iTr=0; iTr<nh; iTr++)
   {
      if (charge[iTr] == 0) continue;
      vMom->SetPxPyPzE(momx[iTr],momy[iTr],momz[iTr],ene[iTr]);

      if (vMom->Perp() < 0.1) continue;
      if (TMath::Abs(vMom->PseudoRapidity()) >= 0.5) continue;

      refMult++;
   }
   return refMult;
}

Int_t BaseReader::GetMultPHENIX(Int_t iEv)
{
   Int_t refMultLeft = 0, refMultRight = 0, refMult = 0;
   if (iEv != -1)
   {
      Long64_t ientry = LoadTree(iEv);
      if (ientry < 0) return -1;
      fChain->GetEntry(iEv);
      // if (Cut(ientry) < 0) continue;
   }

   for (int iTr=0; iTr<nh; iTr++)
   {
      if (charge[iTr] == 0) continue;
      vMom->SetPxPyPzE(momx[iTr],momy[iTr],momz[iTr],ene[iTr]);

      if (vMom->PseudoRapidity() >=  3.1 && vMom->PseudoRapidity() <=  4.0) 
         refMultRight++;
      if (vMom->PseudoRapidity() >=  -4. && vMom->PseudoRapidity() <=  -3.1) 
         refMultLeft++;
      
   }
   if (refMultRight>=2 && refMultLeft>=2) refMult = refMultRight+refMultLeft;
   else refMult = 0;

   return refMult;
}

#endif
