#include <TROOT.h>
#include <TH1F.h>
#include <TFile.h>
#include <iostream>
#include <fstream>

// Additional structs & functions
struct Lim{
  unsigned short L[10];
  unsigned short H[10];
};

// Calculate centrality bins borders
//S----------------------------------------------------------------
Lim CentralityBorders(TH1F* nCh){
  // printf(">>> Centrality file: %s\n", refMultFileName.Data());
  // TFile *cFile = new TFile(refMultFileName.Data(), "READ");
  // TH1F *nCh = (TH1F*) cFile -> Get("hMultPHENIX");

  int    HistLim  = nCh -> GetNbinsX();
  double Integral = nCh -> Integral(1, HistLim);
  double tmpIntegral;
  int    RightLim;
  Lim    lim;
  double efficiency = 0.93;
  double BinWidth   = 10.;
  double Nbins      = 100. / BinWidth;
  double fraction   = (Integral / 100.) * efficiency * BinWidth;

  printf("Integral: %8.0f\n", Integral);
  printf("Centrality bins width: %2.1f, number of bins: %2.0f\n", BinWidth, Nbins);
  printf("Trigger efficiency: %1.2f\n", efficiency);
  printf("Fraction of events in each centrality bin: %8.2f\n", fraction);

  RightLim = HistLim;
  int j = 0;
  for(int i = HistLim; i > 0; --i){
    tmpIntegral = nCh -> Integral(i, RightLim);
    if(tmpIntegral >= fraction){
      printf("[%d,%d]: %8.2f\n", i, RightLim, tmpIntegral);
      lim.H[j] = RightLim;
      lim.L[j] = i;
      ++j;
      RightLim = i - 1;
    }
  }
  // cFile -> Close();
  return lim;
}
//E----------------------------------------------------------------




// Main function
//S----------------------------------------------------------------
void GetCentrality(TString refMultFileName, TString macroName)
{
  // Total number of centrality bins (7 by default for 0-80%)
  const int NcentBins = 7;

  // Get refMult histogram from the file
  TFile *file = new TFile(refMultFileName.Data(),"read");
  TH1F  *hist = (TH1F*) file->Get("hMultPHENIX");

  // Calculate refMult borders for centrality
  Lim lim = CentralityBorders(hist);

  // Write output Takafumi-style macro
  std::ofstream myfile;
  myfile.open(macroName.Data());

  myfile << "Int_t cent8(Int_t mult)" << std::endl;
  myfile << "{" << std::endl;
  myfile << "  Int_t fcent = -1;" << std::endl;
  myfile << "  if (mult >= " << lim.L[0] << ") fcent = 0;" << std::endl;
  for (int i = 1; i < NcentBins; i++)
  {
    myfile << "  if (mult >= " << lim.L[i]
            << " && mult < " << lim.L[i] << ") fcent = "
            << i + 1 << ";" << std::endl;
  }
  myfile << std::endl;
  myfile << "  return fcent;" << std::endl;
  myfile << "}" << std::endl;

  myfile.close();
}
//E----------------------------------------------------------------